/*
Esempio contatore UP/DOWN con PIC16F887, XC8 e MPLAB X
Edits by Vincenzo Dello Iacovo
Lezione originaria:
http://www.settorezero.com/wordpress/corso-programmazione-picmicro-in-c-lezione-8-pilotare-i-display-a-led-a-7-segmenti-in-modalita-multiplex-su-interrupt-del-timer0-realizziamo-un-contatore-up-down/

Attenzione allo schema! I collegamenti sono diversi,
fate riferimento al nuovo articolo.
*/

#include "conf_16f887.h"
#define _XTAL_FREQ 20000000
#include <xc.h>

#define ON  1
#define OFF 0
#define Pressed 0 // Bottone premuto
#define btnUp PORTBbits.RB6 // pulsante incremento contatore
#define btnDn PORTBbits.RB5 // pulsante decremento contatore


// Collegamento Display - Segmenti su PORTD - Catodi su PORTC

#define dispReg PORTD // Banco al quale sono collegati i display
#define displayD PORTCbits.RC5 // attivazione catodo display decine
#define displayU PORTCbits.RC4 // attivazione catodo display unit

//Definizione array per rappresentazione numeri da 0 a 9 

const unsigned char dispnum[]={ 0X3F, //Hex value to display the number 0
                                0X06, //Hex value to display the number 1
                                0X5B, //Hex value to display the number 2
                                0X4F, //Hex value to display the number 3
                                0X66, //Hex value to display the number 4
                                0X6D, //Hex value to display the number 5
                                0X7D, //Hex value to display the number 6
                                0X07, //Hex value to display the number 7
                                0X7F, //Hex value to display the number 8
                                0X6F  //Hex value to display the number 9
                                       }; 


// Variabili
unsigned char counter = 77; // valore del contatore
unsigned char result = 0; // variabile di appoggio per calcoli
unsigned char muxcount = 0; // contatore multiplex
bit actdisplay = 0; // flag che tiene conto del display correntemente visualizzato


// prototipi di funzione
void main(void); // programma principale
void settings(void); //impostazioni porte
void interrupt isr(void); //ISR su overflow TMR0

void main(void)
        {
		settings();
        
        while(1)
			{
            // pulsante "su" premuto
		if (btnUp==Pressed) 
			{
			__delay_ms(100);
			if (btnUp==Pressed) // antibounce
				{
				counter++; // incremento il contatore
				// se il contatore supera 99, lo azzero
				if (counter>99) 
					{
					counter=0;
					}
				}
			}
        // pulsante "giu" premuto
		if (btnDn==Pressed)
			{
			__delay_ms(100);
			if (btnDn==Pressed) // antibounce
				{
				if (counter==0)
					{
					counter=99;
					}
				else
					{
					counter--; // decremento il contatore
					}
				}
			}

            
		}// Fine ciclo continuo
} // fine del main
        
        // INTERRUPT SERVICE ROUTINE
void interrupt isr(void) {
	// questo interrupt si verifica ogni millisecondo come da impostazioni
	if (T0IF)
        {
    	// Ricarico subito il Timer0
		TMR0=100;

		// **************************************************************
		// MULTIPLEXING DISPLAY
		// **************************************************************

		/*
		Visualizzo sui display il valore attuale di "counter".
		Per fare questo devo accendere alternativamente prima un display 
		e la volta successiva l'altro. L'accensione di uno o dell'altro 
		display  comandata dal flag "actdisplay" il quale viene invertito 
		ogni 5ms (ogni 5 interrupt) tramite il contatore "muxcount".
		*/
	
		muxcount++; // incremento il contatore per switch display
	
		/* Controllo quale display accendere:
		actdisplay=0 -> devo accendere il display delle unit
		actdisplay=1 -> devo accendere il display delle decine
		*/
		
		if (actdisplay==0) // devo mostrare solo le unit
			{
			displayU=ON; // accendo il display delle unit
			// mi calcolo quindi la sola unit da mostrare
			result = counter/10; // ottengo la decina (esempio: 28/10 = 2)
			// nota: in C la divisione tra interi restituisce un intero, quindi privo di decimali
			result = counter-(result*10); // ottengo la sola unit
			// se, ad esempio, il contatore era 28, result ora 
			//qui vale 2, quindi dal contatore sottraggo (2*10) e ottengo 8
			}
		else // devo mostrare solo le decine
			{
			result = counter/10; // ottengo la sola decina
			// accendo la decina solo se il risultato  maggiore di zero
			if (result)
				{
				displayD=ON;
				}
			}
	
	/*
	Imposto il registro sul quale sono collegati i display.
	I segmenti da accendere in base al numero da visualizzare sono
	contenuti nell'array "dispnum".
	*/
	
	dispReg=dispnum[result]; 
	
	// in base al valore raggiunto da muxcount, cambio display
	// ogni display rimane acceso in pratica 5millisecondi
	if (muxcount==5)
		{
		muxcount=0; // azzero il contatore
		actdisplay=actdisplay^1; // cambio il display per il prossimo ciclo
		dispReg=0; // azzero il display attualmente visualizzato 
		           //per prevenire l'effetto "ghosting"
		displayU=OFF; // spengo display unit
		displayD=OFF; // spengo display decine
		}

	T0IF=0;// Reset flag interrupt su timer 0
	} // End T0IF
} // End ISR
      
		

void settings(void)
    {
    ANSEL=0; // pins A/D (0-7) come digitali
    ANSELH=0; // pins A/D (8-13) come digitali
    ADCON0bits.ADON=0; // modulo AD spento
    TRISA=0xFF;    //PORTA come ingressi
    TRISB=0xFF;    //PORTB come ingressi
    TRISC=0x00;    //PORTC come uscite
    PORTC=0x00;    //PORTC valore iniziale a 0
    TRISD=0x00;    //PORTD come uscite
    PORTD=0x00;    //PORTD valore iniziale a 0
    TRISE=0xFF;    //PORTF come ingressi
    OPTION_REG=0x44; // resistenze pullup PORTB attivate - clock interno per TMR0 - Prescaler 1:32 assegnato a TMR0
    WPUB=0x60;       // abilito resistenze di pullup su RB6 e RB5
    INTCON=0xA0;     // abilitazione generale interrup e TMR0 - imposto flag a 0
    TMR0=100;       // carico il valore inziale di TMR0
        
    }